@@ -4,14 +4,21 @@ package Array::Iterator::BiDirectional;
 use strict;
 use warnings;
 
-our $VERSION = '0.07'; # VERSION
+our $VERSION = '0.08'; # VERSION
 
 use Array::Iterator;
 our @ISA = qw(Array::Iterator);
 
 sub has_previous {
-	my ($self) = @_;
-	return (($self->_current_index - 1) > 0) ? 1 : 0;
+	my ($self, $n) = @_;
+
+    if(not defined $n) { $n = 1 }
+    elsif(not $n)      { die "has_previous(0) doesn't make sense, did you mean current()?" }
+    elsif($n < 0)      { die "has_previous() with negative argument doesn't make sense, did you mean has_next()?" }
+
+    my $idx = $self->_current_index - $n;
+
+	return ($idx > 0) ? 1 : 0;
 }
 
 sub hasPrevious { my $self = shift; $self->has_previous(@_) }
@@ -34,10 +41,17 @@ sub get_previous {
 sub getPrevious { my $self = shift; $self->get_previous(@_) }
 
 sub look_back {
-	my ($self) = @_;
-    return undef unless (($self->_current_index - 2) > 0);
-        $self->_iterated = 1;
-	return $self->_getItem($self->_iteratee, ($self->_current_index - 2));
+	my ($self, $n) = @_;
+
+    if(not defined $n) { $n = 1 }
+    elsif(not $n)      { die "look_back(0) doesn't make sense, did you mean get_previous()?" }
+    elsif($n < 0)      { die "look_back() with negative argument doesn't make sense, did you mean get_next()?" }
+
+    my $idx = $self->_current_index - ($n + 1);
+
+    return undef unless ($idx > 0);
+    $self->_iterated = 1;
+	return $self->_getItem($self->_iteratee, $idx);
 }
 
 sub lookBack { my $self = shift; $self->look_back(@_) }
@@ -54,7 +68,7 @@ Array::Iterator::BiDirectional - A subclass of Array::Iterator to allow forwards
 
 =head1 VERSION
 
-version 0.07
+version 0.08
 
 =head1 SYNOPSIS
 
@@ -84,9 +98,11 @@ This is a subclass of Array::Iterator, only those methods that have been added a
 
 =over 4
 
-=item B<has_previous>
+=item B<has_previous([$n])>
 
-This method works much like C<isNext> does, it will return true (C<1>) unless the begining of the array has been reached, and false (C<0>) otherwise.
+This method works much like C<hasNext> does, it will return true (C<1>) unless the begining of the array has been reached, and false (C<0>) otherwise.
+
+Optional argument has the same meaning except that it specifies C<$n>th previous element.
 
 =item B<previous>
 
@@ -96,10 +112,12 @@ This method is much like C<next>. It will return the previous item in the iterat
 
 This method is much like C<get_next>. It will return the previous item in the iterator, and return undef if it attempts to reach past the begining of the array.
 
-=item B<look_back>
+=item B<look_back([$n])>
 
 This is the counterpart to C<peek>, it will return the previous items in the iterator, but will not affect the internal counter.
 
+Optional argument has the same meaning except that it specifies C<$n>th previous element.
+
 =back
 
 =head1 BUGS
@@ -133,7 +151,7 @@ Steven Haryanto <stevenharyanto@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2011 by Steven Haryanto.
+This software is copyright (c) 2012 by Steven Haryanto.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
